/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package myTreasure;

import java.awt.Color;
import java.awt.Font;
import java.awt.image.BufferedImage;

public class MyTreasureConstants {

	public static String REGION;
	
	public static Font ORIGINAL_FONT;
	
	public final static String USERLEVELS_GETPHP = "http://www.apo-games.de/myTreasure/get_level.php";
	public final static String USERLEVELS_SAVEPHP = "http://www.apo-games.de/myTreasure/save_level.php";
	
	static {
		try {
			MyTreasureConstants.ORIGINAL_FONT = Font.createFont(Font.TRUETYPE_FONT, MyTreasureConstants.class.getResourceAsStream("/font/pixel.TTF") );
		} catch (Exception e) {
			System.out.println("DAMN");
			e.printStackTrace();
			MyTreasureConstants.ORIGINAL_FONT = new Font(Font.SANS_SERIF, Font.BOLD, 20);
		}
		MyTreasureConstants.REGION = "en";
		try {
			MyTreasureConstants.REGION = System.getProperty("user.language");
		} catch (Exception ex) {
			MyTreasureConstants.REGION = "en";
		}
		MyTreasureConstants.setLanguage(MyTreasureConstants.REGION);
	}
	
	public static BufferedImage iSheet, iWays;
	
	public static final String PROGRAM_NAME = "=== MyTreasure ===";
	public static final double VERSION = 0.01;
	
	public static boolean FPS = false;
	public static boolean FIRST_MAP = true;
	public static boolean FIRST_GAME = true;
	public static boolean FIRST_LEVELCHOOSER = true;
	public static boolean FIRST_LEVELCHOOSER_DRAG = true;
	
	public static final int FPS_THINK = 100;
	public static final int FPS_RENDER = 100;
	
	public static final int WAIT_TIME_RENDER = (int)(1000.0 / (double)FPS_RENDER);
	public static final int WAIT_TIME_THINK = (int)(1000.0 / (double)FPS_THINK);
	
	public static final int GAME_WIDTH = 320;
	public static final int GAME_HEIGHT = 480;
	
	public static final int CHANGE_DRAG = 50;

	public static Font FONT_FPS = MyTreasureConstants.ORIGINAL_FONT.deriveFont(10f).deriveFont(Font.PLAIN);
	
	public static Font FONT_STATISTICS = MyTreasureConstants.ORIGINAL_FONT.deriveFont(20f).deriveFont(Font.BOLD);
	
	public static Font FONT_LEVELCHOOSER = MyTreasureConstants.ORIGINAL_FONT.deriveFont(30f).deriveFont(Font.BOLD);
	
	public static Font fontBig = MyTreasureConstants.ORIGINAL_FONT.deriveFont(33f).deriveFont(Font.BOLD);
	public static Font fontMedium = MyTreasureConstants.ORIGINAL_FONT.deriveFont(28f).deriveFont(Font.BOLD);
	public static Font fontSmall = MyTreasureConstants.ORIGINAL_FONT.deriveFont(17f).deriveFont(Font.BOLD);
	public static Font fontVerySmall = MyTreasureConstants.ORIGINAL_FONT.deriveFont(14f).deriveFont(Font.BOLD);
	
	public static final Color COLOR_DARK = new Color(78, 61, 37);
	public static final Color COLOR_LIGHT = new Color(175, 136, 78);
	public static final Color COLOR_SEPARATOR = new Color(68, 68, 68);
	
	public static final int START_IDLE_TEXT_TIME = 5000;
	public static final int END_IDLE_TEXT_TIME = 9000;
	
	public static final String[] TEXT_IDLE = new String[] {
		"come on",
		"gogo",
		"gaehn",
		"zzZZzz",
	};
	
	public static final String[] DIFFICULTY_ENG = new String[] {
		"camp",
		"temple",
		"pyramid",
		"crypt"
	};
	
	public static final String FIRST_COLLECT_ENG[] = new String[] {
		"Your task:",
		"Collect all Relics."
	};
	
	public static final String FIRST_MOVE_ENG[] = new String[] {
//		"To move the player, rotate the level.",
		"Press left arrow key to",
		"rotate the level clockwise.",
		"Press right arrow key to",
		"rotate the level",
		"counterclockwise."
	};
	
	public static final String[] DIFFICULTY_DE = new String[] {
		"Camp",
		"Tempel",
		"Pyramide",
		"Gruft"
	};
	
	public static final String FIRST_COLLECT_DE[] = new String[] {
		"Dein Ziel:",
		"Sammle alle Schtze ein."
	};
	
	public static final String FIRST_MOVE_DE[] = new String[] {
		"Um den Spieler zu bewegen, rotiere das Level.",
		"Wische mit dem Finger von links nach rechts,",
		"um das Level im Uhrzeigersinn zu drehen",
		"bzw. von rechts nach links, um gegen den",
		"Uhrzeigersinn zu drehen.",
	};
	
	public static String[] DIFFICULTY = MyTreasureConstants.DIFFICULTY_ENG;
	public static String[] FIRST_COLLECT = MyTreasureConstants.FIRST_COLLECT_ENG;
	public static String[] FIRST_MOVE = MyTreasureConstants.FIRST_MOVE_ENG;
	
	/** 
	 * 0 = menu quit
	 * 1 = menu play
	 * 2 = menu userlevels
	 * 3 = menu editor
	 * 4 = menu credits / about
	 * 5 = level Chooser back
	 * 6 = game back
	 * 7 = map back
	 * 8 = game ingame restart
	 * 9 = game ingame next
	 * 10 = game ingame previous
	 * 11 = game result back
	 * 12 = game result restart
	 * 13 = game result next
	 * 14 = credits back
	 * 15 = editor back
	 * 16 = editor size left
	 * 17 = editor size right
	 * 18 = editor test
	 * 19 = editor upload
	*/
	public static final boolean[] BUTTON_MENU = new boolean[] {true, true, false, true, true, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false};
	public static final boolean[] BUTTON_MAP = new boolean[] {false, false, false, false, false, false, false, true, false, false, false, false, false, false, false, false, false, false, false, false, false};
	public static final boolean[] BUTTON_LEVELCHOOSER = new boolean[] {false, false, false, false, false, true, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false};
	public static final boolean[] BUTTON_GAME = new boolean[] {false, false, false, false, false, false, true, false, true, true, true, false, false, false, false, false, false, false, false, false, false};
	public static final boolean[] BUTTON_EDITOR = new boolean[] {false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, true, true, true, true, false, false};
	public static final boolean[] BUTTON_CREDITS = new boolean[] {false, false, false, false, false, false, false, false, false, false, false, false, false, false, true, false, false, false, false, false, false};

	public static final void setLanguage(final String region) {
		if ((region != null) && (region.equals("de"))) {
			MyTreasureConstants.DIFFICULTY = MyTreasureConstants.DIFFICULTY_DE;
			MyTreasureConstants.FIRST_COLLECT = MyTreasureConstants.FIRST_COLLECT_DE;
			MyTreasureConstants.FIRST_MOVE = MyTreasureConstants.FIRST_MOVE_DE;
		} else {
			MyTreasureConstants.DIFFICULTY = MyTreasureConstants.DIFFICULTY_ENG;
			MyTreasureConstants.FIRST_COLLECT = MyTreasureConstants.FIRST_COLLECT_ENG;
			MyTreasureConstants.FIRST_MOVE = MyTreasureConstants.FIRST_MOVE_ENG;
		}
	}
}
