/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package myTreasure.entity;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.IOException;
import java.util.Map;

import myTreasure.MyTreasureConstants;
import myTreasure.game.MyTreasurePanel;

import org.apogames.ApoIO;

public class MyTreasureIO extends ApoIO {

	private final MyTreasurePanel panel;
	
	public MyTreasureIO(final MyTreasurePanel panel) {
		this.panel = panel;
	}
	
	@Override
	public boolean writeLevel(DataOutputStream data) throws IOException {
		data.writeInt(this.panel.getSolvedLevel().size());
		for (Map.Entry<String, Integer> entry : this.panel.getSolvedLevel().entrySet()) {
		    String key = entry.getKey();
		    Integer value = entry.getValue();
		    data.writeUTF(key);
		    data.writeInt(value);
		}
		data.writeInt(this.panel.getCurSkulls());
		data.writeBoolean(MyTreasureConstants.FIRST_MAP);
		data.writeBoolean(MyTreasureConstants.FIRST_LEVELCHOOSER);
		data.writeBoolean(MyTreasureConstants.FIRST_LEVELCHOOSER_DRAG);
		data.writeBoolean(MyTreasureConstants.FIRST_GAME);
		return false;
	}

	@Override
	public boolean readLevel(DataInputStream data) throws EOFException, IOException {
		int skulls = 0;
		int count = data.readInt();
		for (int i = 0; i < count; i++) {
			String level = data.readUTF();
			int levelSkulls = data.readInt();
			this.panel.getSolvedLevel().put(level, levelSkulls);
			skulls += levelSkulls;
		}
		try {
			this.panel.setCurSkulls(data.readInt());
			this.panel.setOldLevel(this.panel.getCurSkulls());
			MyTreasureConstants.FIRST_MAP = data.readBoolean();
			MyTreasureConstants.FIRST_LEVELCHOOSER = data.readBoolean();
			MyTreasureConstants.FIRST_LEVELCHOOSER_DRAG = data.readBoolean();
			MyTreasureConstants.FIRST_GAME = data.readBoolean();
		} catch (Exception ex) {
			this.panel.setCurSkulls(skulls);
		}
		return false;
	}

}
