/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package myTreasure.game;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import myTreasure.MyTreasureConstants;

public class MyTreasureEditor extends MyTreasureModel {

	public static final int ENTITY_WALL = 0;
	public static final int ENTITY_FREE = 1;
	public static final int ENTITY_PLAYER = 2;
	public static final int ENTITY_SKULL = 3;
	public static final int ENTITY_BLOCK = 4;
	
	public static final String BACK = "back";
	public static final String SIZE_LEFT = "left";
	public static final String SIZE_RIGHT = "right";
	public static final String TEST = "test";
	public static final String UPLOAD = "upload";
	public static final String SOLVE = "solve";
	
	private BufferedImage iBackground, light;
	
	private int[][] level;
	
	private int size = 6;
	
	private int curChoose;
	
	private int steps;
	
	private int startX, startY;
	
	private int time;
	private String uploadString;
	
	private Thread t;
	
	public MyTreasureEditor(final MyTreasurePanel game) {
		super(game);
	}

	@Override
	public void init() {
		if (this.iBackground == null) {
			this.makeScreenBackground();
		}
		if (this.light == null) {
			this.light = new BufferedImage(64, 64, BufferedImage.TYPE_INT_ARGB_PRE);
			Graphics2D g = this.light.createGraphics();
			g.setColor(new Color(255,200,0,4));
			for (int i = 5; i < 30; i += 1) {
				g.fillOval(0 + i, 0 + i, 64 - i*2, 64 - i*2);
			}
			g.dispose();
		}
		if (this.level == null) {
			this.level = new int[6][6];
			this.makeStandardLevel();
			
			this.curChoose = MyTreasureEditor.ENTITY_BLOCK;
		}
		this.time = 0;
		this.setCanBeTest();
	}
	
	public void setSteps(final int steps) {
		if (this.steps > steps) {
			this.steps = steps;
		}
		
		System.out.println(this.getLevelString());
	}
	
	public void setVisibleUpload(final boolean visible) {
		this.getGame().getButtons()[19].setBVisible(visible);	
	}	
	
	public void makeStandardLevel() {
		for (int y = 1; y < this.level.length - 1; y++) {
			for (int x = 1; x < this.level[0].length - 1; x++) {
				this.level[y][x] = 1;
			}
		}
		this.level[this.level.length - 2][2] = 2;
		this.level[this.level.length - 2][1] = 4;
		this.level[1][this.level[0].length - 3] = 3;
		
		this.startX = MyTreasureConstants.GAME_WIDTH/2 - this.level[0].length * 16;
		this.startY = MyTreasureConstants.GAME_HEIGHT/2 - this.level.length * 16 - 32;
	}
	
	public void setCanBeTest() {
		int player = 0;
		int skull = 0;
		
		for (int y = 0; y < this.level.length; y += 1) {
			for (int x = 0; x < this.level[0].length; x += 1) {
				if (this.level[y][x] == 2) player += 1;
				if (this.level[y][x] == 3) skull += 1;
			}
		}
		
		if ((player == 1) && (skull == 1)) {
			this.getGame().getButtons()[18].setBVisible(true);
			this.getGame().getButtons()[20].setBVisible(true);
		} else {
			this.getGame().getButtons()[18].setBVisible(false);
			this.getGame().getButtons()[20].setBVisible(false);
		}
	}
	
	public String getLevelString() {
		String levelString = "";
		levelString = String.valueOf(this.size);
		if (levelString.length() < 2) {
			levelString = "0" + levelString;
		}
		levelString = levelString + levelString;
		for (int y = 0; y < this.level.length; y += 1) {
			for (int x = 0; x < this.level[0].length; x += 1) {
				levelString += String.valueOf(this.level[y][x]);
			}
		}
		String stepsString = String.valueOf(this.steps);
		
		if (stepsString.length() < 2) {
			stepsString = "0" + stepsString;
		}
		
		levelString += stepsString;
		
		stepsString = String.valueOf(this.steps + 4);
		if (stepsString.length() < 2) {
			stepsString = "0" + stepsString;
		}
		
		levelString += stepsString;
		
		return levelString;
	}
	
	public void makeScreenBackground() {
		this.iBackground = new BufferedImage(320,480,BufferedImage.TYPE_INT_RGB);
		Graphics2D g = this.iBackground.createGraphics();
		for (int y = 0; y < 16; y += 1) {
			for (int x = 0; x < 11; x += 1) {
				g.drawImage(this.getGame().getISheet().getSubimage((int)(Math.random() * 4) * 32, 3 * 32, 32, 32), (x-1) * 32, (y) * 32, null);
			}				
			if (y == 14) {
				for (int i = 0; i < 3; i++) {
					g.drawImage(this.getGame().getISheet().getSubimage(128, 64, 64, 32), 0, 448 - i * 32, null);
					g.drawImage(this.getGame().getISheet().getSubimage(128, 96, 63, 32), 64, 448 - i * 32, null);
					g.drawImage(this.getGame().getISheet().getSubimage(132, 64, 60, 32), 127, 448 - i * 32, null);
					g.drawImage(this.getGame().getISheet().getSubimage(128, 96, 63, 32), 187, 448 - i * 32, null);
					g.drawImage(this.getGame().getISheet().getSubimage(132, 64, 6, 32), 250, 448 - i * 32, null);
					g.drawImage(this.getGame().getISheet().getSubimage(128, 96, 64, 32), 256, 448 - i * 32, null);
				}

				g.drawImage(this.getGame().getISheet().getSubimage(128, 64, 64, 32), 0, 0, null);
				g.drawImage(this.getGame().getISheet().getSubimage(128, 96, 63, 32), 64, 0, null);
				g.drawImage(this.getGame().getISheet().getSubimage(132, 64, 60, 32), 127, 0, null);
				g.drawImage(this.getGame().getISheet().getSubimage(128, 96, 63, 32), 187, 0, null);
				g.drawImage(this.getGame().getISheet().getSubimage(132, 64, 6, 32), 250, 0, null);
				g.drawImage(this.getGame().getISheet().getSubimage(128, 96, 64, 32), 256, 0, null);
			}
		}
		g.dispose();
	}

	public void keyPressed(int keyCode, char keyCharacter) {
	}
	
	@Override
	public void keyButtonReleased(int button, char character) {
	}

	@Override
	public void mouseButtonFunction(String function) {
		if (function.equals(MyTreasureEditor.BACK)) {
			this.getGame().setMenu();
		} else if (function.equals(MyTreasureEditor.SIZE_LEFT)) {
			if (this.size > 5) {
				this.size -= 1;
				this.level = new int[this.size][this.size];
				this.makeStandardLevel();
				this.steps = 95;
			}
		} else if (function.equals(MyTreasureEditor.SIZE_RIGHT)) {
			if (this.size < 10) {
				this.size += 1;
				this.level = new int[this.size][this.size];
				this.makeStandardLevel();
				this.steps = 95;
			}
		} else if (function.equals(MyTreasureEditor.TEST)) {
			this.getGame().setGame(false, true, 0, this.getLevelString());
			System.out.println(this.getLevelString());
		} else if (function.equals(MyTreasureEditor.UPLOAD)) {
			this.upload();
		} else if (function.equals(MyTreasureEditor.SOLVE)) {
			this.setSolutionString("");
			this.getGame().solveLevel(this.getLevelString());
		}
	}
	
	public void setSolutionString(final String solution) {
		this.time = 5000;
		if (solution.length() == 0) {
			this.uploadString = "try to solve the level";
		} else {
			this.uploadString = "level solved in "+solution.length()+" steps";
			this.steps = solution.length();
			this.setVisibleUpload(true);
		}
	}
	
	public void upload() {
		this.time = 5000;
		this.uploadString = "uploading ...";
		this.setVisibleUpload(false);
		this.t = new Thread(new Runnable() {
			public void run() {
				MyTreasureEditor.this.uploadString();
			}
 		});
		this.t.start();
	}
	
	public void uploadString() {
		boolean bUpload = this.getGame().getUserlevels().addLevel(this.getLevelString());
		if (bUpload) {
			this.uploadString = "upload successfully";
			this.getGame().loadUserlevels();
			this.time = 2000;
		} else {
			this.uploadString = "upload failed";
			this.time = 2000;
		}
	}

	@Override
	public void mouseButtonReleased(int x, int y, boolean bRight) {
		if ((y >= MyTreasureConstants.GAME_HEIGHT - 64) && (y <= MyTreasureConstants.GAME_HEIGHT - 32) && 
			(x >= 128)) {
			for (int i = 0; i < 5; i++) {
				if ((x >= 128 + i * 40) && (x <= 160 + i * 40)) {
					this.curChoose = i;
					break;
				}
			}
		}
		if ((y >= this.startY) && (y < this.startY + this.size * 32) &&
			(x >= this.startX) && (x < this.startX + this.size * 32)) {
			int curX = (x - this.startX) / 32;
			int curY = (y - this.startY) / 32;
			int curChoose = this.curChoose;
			if (bRight) {
				curChoose = ENTITY_FREE;
			}
			if ((curChoose == 2) || (curChoose == 3)) {
				if ((curX < 1) || (curY < 1) || (curX >= this.level[0].length - 1) || (curY >= this.level.length - 1)) {
					return;
				}
			}
			if (this.level[curY][curX] != curChoose) {
				if ((curChoose == MyTreasureEditor.ENTITY_SKULL) || (curChoose == MyTreasureEditor.ENTITY_PLAYER)) {
					for (int lY = 0; lY < this.level.length; lY += 1) {
						for (int lX = 0; lX < this.level[0].length; lX += 1) {
							if (this.level[lY][lX] == curChoose) {
								this.level[lY][lX] = MyTreasureEditor.ENTITY_FREE;
							}
						}
					}
				}
				this.level[curY][curX] = curChoose;
				this.steps = 95;
				this.setCanBeTest();
				this.setVisibleUpload(false);
			}
		}
	}

	public boolean mouseMoved(int x, int y) {
		return false;
	}
	
	public boolean mouseDragged(int x, int y, boolean bRight) {
		if ((y >= this.startY) && (y < this.startY + this.size * 32) &&
			(x >= this.startX) && (x < this.startX + this.size * 32)) {
			this.mouseButtonReleased(x, y, bRight);
		}
		return false;
	}
	
	public boolean mousePressed(int x, int y, boolean bRight) {
		return false;
	}
	
	public void mouseWheelChanged(int changed) {
		this.curChoose += changed;
		if (this.curChoose < 0) {
			this.curChoose = ENTITY_BLOCK;
		} else if (this.curChoose > ENTITY_BLOCK) {
			this.curChoose = 0;
		}
	}
	
	@SuppressWarnings("deprecation")
	@Override
	public void think(int delta) {
		if (this.time > 0) {
			this.time -= delta;
			if (this.time <= 0) {
				if (this.uploadString.equals("uploading ...")) {
					this.t.stop();
				}
			}
		}
	}

	@Override
	public void render(Graphics2D g) {
		g.drawImage(this.iBackground, 0, 0, null);
		
		for (int y = 0; y < this.level.length; y += 1) {
			for (int x = 0; x < this.level[0].length; x += 1) {
				if (this.level[y][x] == 3) {
					g.drawImage(this.light, this.startX + (x) * 32 - 16, this.startY +(y) * 32 - 16, null);
					g.drawImage(this.getGame().getISheet().getSubimage(0 * 32, 1 * 32, 32, 32), this.startX + (x) * 32, this.startY + (y) * 32, null);
				}
				if (this.level[y][x] == 4) {
					g.drawImage(this.getGame().getISheet().getSubimage(2 * 32, 0 * 32, 32, 32), this.startX + (x) * 32, this.startY + (y) * 32, null);
				}
				if (this.level[y][x] == 0) {
					g.drawImage(this.getGame().getISheet().getSubimage(this.level[y][x] * 32, 2 * 32, 32, 32), this.startX + (x) * 32, this.startY + (y) * 32, null);
				}
				if (this.level[y][x] == 2) {
					int curX = 0;
					int curY = 0;
					g.drawImage(this.getGame().getISheet().getSubimage(curX * 32, curY * 32, 32, 32), this.startX + (x) * 32, this.startY + (y) * 32, null);	
				}
				if (this.level[y][x] == 1) {
					g.setColor(MyTreasureConstants.COLOR_LIGHT);
					g.drawRect(this.startX + (x) * 32, this.startY + (y) * 32, 32, 32);
				}
			}
		}
		
		g.setColor(MyTreasureConstants.COLOR_SEPARATOR);
		g.setFont(MyTreasureConstants.FONT_STATISTICS);
		
		String s = "size";
		int w = g.getFontMetrics().stringWidth(s);
		g.drawString(s, 60 - w/2, MyTreasureConstants.GAME_HEIGHT - 72);
		
		s = String.valueOf(this.size);
		w = g.getFontMetrics().stringWidth(s);
		g.drawString(s, 60 - w/2, MyTreasureConstants.GAME_HEIGHT - 38);
		
		s = "level objects";
		w = g.getFontMetrics().stringWidth(s);
		g.drawString(s, 220 - w/2, MyTreasureConstants.GAME_HEIGHT - 72);
		
		g.setColor(MyTreasureConstants.COLOR_SEPARATOR);
		g.fillRect(120, MyTreasureConstants.GAME_HEIGHT - 94, 2, 92);
		
		// block
		g.drawImage(this.getGame().getISheet().getSubimage(0 * 32, 2 * 32, 32, 32), 128, MyTreasureConstants.GAME_HEIGHT - 64, null);
		// free
		g.drawImage(this.getGame().getISheet().getSubimage(0 * 32, 3 * 32, 32, 32), 168, MyTreasureConstants.GAME_HEIGHT - 64, null);
		// player
		g.drawImage(this.getGame().getISheet().getSubimage(0 * 32, 0 * 32, 32, 32), 208, MyTreasureConstants.GAME_HEIGHT - 64, null);
		// skull
		g.drawImage(this.getGame().getISheet().getSubimage(0 * 32, 1 * 32, 32, 32), 248, MyTreasureConstants.GAME_HEIGHT - 64, null);
		// block
		g.drawImage(this.getGame().getISheet().getSubimage(2 * 32, 0 * 32, 32, 32), 288, MyTreasureConstants.GAME_HEIGHT - 64, null);
		
		g.setColor(new Color(255, 0, 0));
		g.drawRect(128 + 40 * this.curChoose, MyTreasureConstants.GAME_HEIGHT - 64, 31, 32);
		
		if (this.time > 0) {
			g.setColor(new Color(255, 255, 255));
			w = g.getFontMetrics().stringWidth(this.uploadString);
			int x = 128 - w/2 - 5;
			if (x < 0) {
				x = 0;
			}
			g.fillRect(x + 1, 32, w + 8, 24);
			g.setColor(new Color(0, 0, 0));
			g.drawRect(x + 1, 32, w + 8, 24);
			g.drawString(this.uploadString, x + 5, 52);
		}
		
	}

}

