/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package myTreasure.game;

import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import org.apogames.ApoConstants;
import org.apogames.entity.ApoButton;

import myTreasure.MyTreasureConstants;
import myTreasure.entity.MyTreasureTutorial;
import myTreasure.entity.MyTreasureTutorialHelp;
import myTreasure.userlevels.MyTreasureLevels;

public class MyTreasureLevelChooser extends MyTreasureModel {

	public static final String BACK = "back";
	
	public static final int EASY = 0;
	public static final int MEDIUM = 1;
	public static final int HARD = 2;
	public static final int VERYHARD = 3;
	
	public static int FREE = 5;
	
	private BufferedImage iBackground;
	
	private int difficulty;
	
	private ApoButton[] levels;
	
	private float curChange = 0;
	private int curStart = 0;
	private int curI = 0;
	private int min = 0;
	private int max = 20;
	
	private int mousePressedX, mousePressedY;
	
	private int curTouchX;
	
	private boolean bHandCursor;
	
	private MyTreasureTutorial tutorial;
	
	public MyTreasureLevelChooser(final MyTreasurePanel game) {
		super(game);
		
		this.difficulty = MyTreasureLevelChooser.EASY;
		this.bHandCursor = false;
		
		if (ApoConstants.B_APPLET) {
			MyTreasureLevelChooser.FREE = 41;
		}
		
	}

	@Override
	public void init() {
		if (this.iBackground == null) {
			this.iBackground = new BufferedImage(MyTreasureConstants.GAME_WIDTH, MyTreasureConstants.GAME_HEIGHT, BufferedImage.TYPE_INT_RGB);
			Graphics2D g = this.iBackground.createGraphics();
			g.drawImage(this.getGame().getISheet().getSubimage(this.getGame().getISheet().getWidth() - MyTreasureConstants.GAME_WIDTH, 0, MyTreasureConstants.GAME_WIDTH, MyTreasureConstants.GAME_HEIGHT), 0, 0, null);
			
			g.drawImage(this.getGame().getISheet().getSubimage(0, 96 * 4, 76 * 4, 8 * 4), 2 * 4, 58, null);
			
			g.dispose();
		}
		if (this.levels == null) {
			this.levels = new ApoButton[100];
			int curX = 6 * 4;
			int curY = 24 * 4;
			for (int i = 0; i < 20; i++) {
				this.levels[i + 0 * 20] = new ApoButton(null, curX, curY, 16 * 4, 16 * 4, "");
				this.levels[i + 1 * 20] = new ApoButton(null, curX + 17 * 16, curY, 16 * 4, 16 * 4, "");
				this.levels[i + 2 * 20] = new ApoButton(null, curX + 17 * 32, curY, 16 * 4, 16 * 4, "");
				this.levels[i + 3 * 20] = new ApoButton(null, curX + 17 * 48, curY, 16 * 4, 16 * 4, "");
				this.levels[i + 4 * 20] = new ApoButton(null, curX + 17 * 64, curY, 16 * 4, 16 * 4, "");
				
				curX += 17 * 4;
				if (curX > 280) {
					curX = 6 * 4;
					curY += 18 * 4;
				}
			}
		}
		
		if (MyTreasureConstants.FIRST_LEVELCHOOSER) {
			MyTreasureTutorialHelp[] points = new MyTreasureTutorialHelp[4];
			points[0] = new MyTreasureTutorialHelp(144, 224, 64, 224, 500);
			points[1] = new MyTreasureTutorialHelp(48, 113, 64, 224, 100);
			points[2] = new MyTreasureTutorialHelp(48, 113, 96, 224, 1000);
			points[3] = new MyTreasureTutorialHelp(48, 113, 64, 224, 600);
			this.tutorial = new MyTreasureTutorial(points);
		} else {
			this.tutorial = null;
		}
	}
	
	public int getCurMax() {
		int free = MyTreasureLevelChooser.FREE;
		for (int i = 0; i < this.levels.length && i < this.max - this.min; i++) {
			boolean bSolved = this.getGame().getSolvedLevel().containsKey(MyTreasureLevels.getLevel(this.curI + i));
			if (!bSolved) {
				free -= 1;
				if (free <= 0) {
					return i;
				}
			}
		}
		return this.max - this.min;
	}

	public int getMin() {
		return this.min;
	}

	public int getCurI() {
		return this.curI;
	}

	public int getDifficulty() {
		return this.difficulty;
	}

	public void setDifficulty(final int difficulty, final int level) {
		this.difficulty = difficulty;
		this.curChange = 0;
		this.curStart = level / 20;
		
		if (this.difficulty == MyTreasureLevelChooser.EASY) {
			this.min = 0;
			this.max = 40;
			this.curI = 0; 
		}
		if (this.difficulty == MyTreasureLevelChooser.MEDIUM) {
			this.min = 40;
			this.max = 80;
			this.curI = 40; 
		}
		
		if (this.difficulty == MyTreasureLevelChooser.HARD) {
			this.min = 80;
			this.max = 120;
			this.curI = 80; 
		}
		if (this.difficulty == MyTreasureLevelChooser.VERYHARD) {
			this.min = 120;
			this.max = 150;
			this.curI = 120; 
		}
		
		if ((!MyTreasureConstants.FIRST_LEVELCHOOSER) && (level < 20) && (level + MyTreasureLevelChooser.FREE >= 20) && (MyTreasureConstants.FIRST_LEVELCHOOSER_DRAG)) {
			MyTreasureTutorialHelp[] points = new MyTreasureTutorialHelp[5];
			points[0] = new MyTreasureTutorialHelp(144, 224, 64, 224, 0);
			points[1] = new MyTreasureTutorialHelp(289, 65, 64, 224, 0);
			points[2] = new MyTreasureTutorialHelp(289, 65, 96, 224, 200);
			points[3] = new MyTreasureTutorialHelp( 40, 65, 96, 224, 50);
			points[4] = new MyTreasureTutorialHelp( 40, 65, 64, 224, 500);
			this.tutorial = new MyTreasureTutorial(points);
		} else if ((!MyTreasureConstants.FIRST_LEVELCHOOSER) && (level >= 20) && (MyTreasureConstants.FIRST_LEVELCHOOSER_DRAG)) {
			MyTreasureTutorialHelp[] points = new MyTreasureTutorialHelp[5];
			points[0] = new MyTreasureTutorialHelp(144, 224, 64, 224, 0);
			points[1] = new MyTreasureTutorialHelp( 40, 65, 64, 224, 0);
			points[2] = new MyTreasureTutorialHelp( 40, 65, 96, 224, 200);
			points[3] = new MyTreasureTutorialHelp(289, 65, 96, 224, 50);
			points[4] = new MyTreasureTutorialHelp(289, 65, 64, 224, 500);
			this.tutorial = new MyTreasureTutorial(points);
		}
	}

	public void keyPressed(int keyCode, char keyCharacter) {
	}
	
	@Override
	public void keyButtonReleased(int button, char character) {
	}

	@Override
	public void mouseButtonFunction(String function) {
		if (function.equals(MyTreasureMap.BACK)) {
			this.getGame().setMap();
		}
	}

	@Override
	public void mouseButtonReleased(int x, int y, boolean bRight) {
		if (Math.abs(x - this.mousePressedX) > MyTreasureConstants.CHANGE_DRAG) {
			if (this.mousePressedX > x) {
				if (this.curStart * 20 + 20 + this.min < this.max) {
					this.curChange = 17 * 16 + (this.curTouchX - this.mousePressedX);
					this.curStart += 1;
				}
			}
			if (this.mousePressedX < x) {
				if (this.curStart * 20 - 20 + this.min >= this.min) {
					this.curChange = -17 * 16 + (this.curTouchX - this.mousePressedX);
					this.curStart -= 1;
				}
			}
		} else if (Math.abs(x - this.mousePressedX) <= 2) {
			int free = MyTreasureLevelChooser.FREE;
			for (int i = 0; i < this.levels.length && i < this.max - this.min; i++) {
				boolean bSolved = this.getGame().getSolvedLevel().containsKey(MyTreasureLevels.getLevel(this.curI + i));
				if (!bSolved) {
					free -= 1;
				}
				if ((this.levels[i].getX() <= x + this.curStart * 272) && (this.levels[i].getX() + 64 >= x + this.curStart * 272) &&
					(this.levels[i].getY() <= y) && (this.levels[i].getY() + 64 >= y)) {
					if ((free > 0) || (bSolved)) {
						if (MyTreasureConstants.FIRST_LEVELCHOOSER) {
							MyTreasureConstants.FIRST_LEVELCHOOSER = false;
							if (this.tutorial != null) {
								this.tutorial.setVisible(false);
							}
						}
						this.getGame().setGame(false, false, i, "");
					}
					return;
				}
			}
		} else if (this.curTouchX != 0){
			this.curChange = (this.curTouchX - this.mousePressedX);
		}
		this.curTouchX = 0;
		this.mousePressedX = 0;
	}

	public boolean mouseMoved(int x, int y) {
		int free = MyTreasureLevelChooser.FREE;
		for (int i = 0; i < this.levels.length && i < this.max - this.min; i++) {
			boolean bSolved = this.getGame().getSolvedLevel().containsKey(MyTreasureLevels.getLevel(this.curI + i));
			if (!bSolved) {
				free -= 1;
			}
			if ((this.levels[i].getX() <= x + this.curStart * 272) && (this.levels[i].getX() + 64 >= x + this.curStart * 272) &&
				(this.levels[i].getY() <= y) && (this.levels[i].getY() + 64 >= y)) {
				if ((free > 0) || (bSolved)) {
					if (!this.bHandCursor) {
						this.bHandCursor = true;
						this.getGame().getScreen().getComponent().setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
					}
				}
				return false;
			}
		}
		if ((this.bHandCursor) && (y > 20 * 4)) {
			this.bHandCursor = false;
			this.getGame().getScreen().getComponent().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
		return false;
	}
	
	public boolean mouseDragged(int x, int y, boolean bRight) {
		if (this.mousePressedX > x) {
			if (this.curStart * 20 + 20 + this.min < this.max) {
				this.curTouchX = x;
			}
		}
		if (this.mousePressedX < x) {
			if (this.curStart * 20 - 20 + this.min >= this.min) {
				this.curTouchX = x;
			}
		}
		return false;
	}
	
	public boolean mousePressed(int x, int y, boolean bRight) {
		this.mousePressedX = x;
		this.mousePressedY = y;
		this.curTouchX = x;
		return false;
	}
	
	@Override
	public void think(int delta) {
		if (this.curChange > 0) {
			this.curChange -= 4;
		}
		
		if (this.curChange < 0) {
			this.curChange += 4;
		}
		
		if (this.tutorial != null) {
			if (this.tutorial.isVisible()) {
				this.tutorial.think(delta);
				if (!this.tutorial.isVisible()) {
					if (MyTreasureConstants.FIRST_LEVELCHOOSER) {
						MyTreasureConstants.FIRST_LEVELCHOOSER = false;
					} else {
						MyTreasureConstants.FIRST_LEVELCHOOSER_DRAG = false;
					}
				}
			}
		}
	}

	@Override
	public void render(Graphics2D g) {
		g.drawImage(this.iBackground, 0, 0, null);
		
		g.setFont(MyTreasureConstants.FONT_LEVELCHOOSER);
		g.setColor(MyTreasureConstants.COLOR_DARK);
		String s = MyTreasureConstants.DIFFICULTY[this.difficulty];
		int w = g.getFontMetrics().stringWidth(s);
		g.drawString(s, MyTreasureConstants.GAME_WIDTH*1/2 - w/2, 48);
		
		if ((this.max - this.min - 1) / 20 > 0) {
			g.setFont(MyTreasureConstants.fontSmall);
			s = " ("+(this.curStart + 1) + " / " + ((this.max - this.min)/21 + 1)+")";
			g.drawString(s, MyTreasureConstants.GAME_WIDTH*1/2 + w/2, 46);	
		}
		
		int free = MyTreasureLevelChooser.FREE;
		g.setClip(16, 96, 288, 360);
		for (int i = 0; i < this.levels.length && i < this.max - this.min; i++) {
			boolean bSolved = this.getGame().getSolvedLevel().containsKey(MyTreasureLevels.getLevel(this.curI + i));
			if (!bSolved) {
				free -= 1;
			}
			int change = 0;
			if (this.curTouchX > 0) {
				change = -(this.curTouchX - this.mousePressedX);
			}
			if ((this.levels[i].getX() + 64 > 20 - this.curChange + this.curStart * 272 + change) && (this.levels[i].getX() < 20 - this.curChange + (this.curStart + 1) * 272 + change)) {
				int add = 0;
				if ((Math.abs(this.curTouchX - this.mousePressedX) <= 2) && (this.levels[i].intersects(this.mousePressedX - this.curChange + this.curStart * 272 + change, this.mousePressedY))) {
					add = 2;
				}
				if ((free > 0) || (bSolved)) {
					g.drawImage(this.getGame().getISheet().getSubimage(16 * 4, 80 * 4, 64, 64), (int)(this.levels[i].getX() - change + this.curChange - this.curStart * 272), (int)(this.levels[i].getY() + add), null);
				} else {
					g.drawImage(this.getGame().getISheet().getSubimage(0 * 4, 80 * 4, 64, 64), (int)(this.levels[i].getX() - change + this.curChange - this.curStart * 272), (int)(this.levels[i].getY() + add), null);
				}
				g.setColor(MyTreasureConstants.COLOR_LIGHT);
				g.setFont(MyTreasureConstants.fontBig);
				s = String.valueOf(i + 1);
				w = g.getFontMetrics().stringWidth(s);
				g.drawString(s, (int)(this.levels[i].getX() - change + this.curChange - this.curStart * 272 + 32) - w/2, (int)(this.levels[i].getY() + 33 + add));
				
				if (bSolved) {
					int skulls = this.getGame().getSolvedLevel().get(MyTreasureLevels.getLevel(this.curI + i));
					g.drawImage(this.getGame().getISheet().getSubimage(8 * 4, 120 * 4, 12, 12), (int)(this.levels[i].getX() - change + this.curChange - this.curStart * 272 + 2 * 4), (int)(this.levels[i].getY() + 10 * 4 + add), null);
					if (skulls > 1) {
						g.drawImage(this.getGame().getISheet().getSubimage(12 * 4, 120 * 4, 12, 12), (int)(this.levels[i].getX() - change + this.curChange - this.curStart * 272 + 6 * 4), (int)(this.levels[i].getY() + 10 * 4 + add), null);
						if (skulls > 2) {
							g.drawImage(this.getGame().getISheet().getSubimage(16 * 4, 120 * 4, 12, 12), (int)(this.levels[i].getX() - change + this.curChange - this.curStart * 272 + 10 * 4), (int)(this.levels[i].getY() + 10 * 4 + add), null);	
						}
					}
				}
			}
		}
		g.setClip(0, 0, MyTreasureConstants.GAME_WIDTH, MyTreasureConstants.GAME_HEIGHT);
		
		if (this.tutorial != null) {
			if (this.tutorial.isVisible()) {
				this.tutorial.render(g);
			}
		}
	}

}

