/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package myTreasure.game;

import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import org.apogames.ApoConstants;

import myTreasure.MyTreasureConstants;
import myTreasure.entity.MyTreasureMapPlayer;
import myTreasure.entity.MyTreasureParticle;
import myTreasure.entity.MyTreasureTutorial;
import myTreasure.entity.MyTreasureTutorialHelp;

public class MyTreasureMap extends MyTreasureModel {

	public static final String BACK = "back";
	
	private final int PARTICLE_SIZE = 50;
	
	private int[] NEEDED_SKULLS = new int[] {
		40,
		140,
		280
	};
	
	private BufferedImage iBackground;
	
	private MyTreasureMapPlayer player;
	
	private MyTreasureTutorial tutorial;
	
	private ArrayList<MyTreasureParticle> particles;
	
	private int oldLevel;
	
	/** boolean Variable, ob gerade ein Handcursor angezeigt wird oder nicht */
	private boolean bHandCursor;
	
	public MyTreasureMap(final MyTreasurePanel game) {
		super(game);
		
		if (ApoConstants.B_APPLET) {
			this.NEEDED_SKULLS = new int[] {
				0,
				0,
				0
			};
		}
	}

	@Override
	public void init() {
		if (this.iBackground == null) {
			this.iBackground = new BufferedImage(MyTreasureConstants.GAME_WIDTH, MyTreasureConstants.GAME_HEIGHT, BufferedImage.TYPE_INT_RGB);
			Graphics2D g = this.iBackground.createGraphics();
			g.drawImage(this.getGame().getISheet().getSubimage(this.getGame().getISheet().getWidth() - 2 * MyTreasureConstants.GAME_WIDTH, 0, MyTreasureConstants.GAME_WIDTH, MyTreasureConstants.GAME_HEIGHT), 0, 0, null);
			g.dispose();
		}
		if (this.player == null) {
			this.player = new MyTreasureMapPlayer();
		}
		
		if (this.tutorial == null) {
			MyTreasureTutorialHelp[] points = new MyTreasureTutorialHelp[4];
			points[0] = new MyTreasureTutorialHelp(144, 224, 64, 224, 500);
			points[1] = new MyTreasureTutorialHelp(40, 390, 64, 224, 0);
			points[2] = new MyTreasureTutorialHelp(40, 390, 96, 224, 2000);
			points[3] = new MyTreasureTutorialHelp(40, 390, 64, 224, 500);
			this.tutorial = new MyTreasureTutorial(points);
		}
		if (this.particles == null) {
			this.particles = new ArrayList<MyTreasureParticle>();
		}
		for (int i = 0; i < this.NEEDED_SKULLS.length; i++) {
			if ((this.oldLevel < this.NEEDED_SKULLS[i]) && (this.getGame().getCurSkulls() > this.NEEDED_SKULLS[i])) {
				this.particles.clear();
				int x = 178;
				int y = 446;
				if (i == 1) {
					x = 222;
					y = 264;
				} else if (i == 2) {
					x = 216;
					y = 148;
				}
				for (int p = 0; p < this.PARTICLE_SIZE; p++) {
					this.particles.add(new MyTreasureParticle(x, y, 4, 4));
				}
				break;
			}
		}
		
		this.oldLevel = this.getGame().getCurSkulls();
	}

	public void setOldLevel(final int oldLevel) {
		this.oldLevel = oldLevel;
	}

	public void keyPressed(int keyCode, char keyCharacter) {
	}
	
	@Override
	public void keyButtonReleased(int button, char character) {
	}

	@Override
	public void mouseButtonFunction(String function) {
		if (function.equals(MyTreasureMap.BACK)) {
			this.getGame().setMenu();
		}
	}

	@Override
	public void mouseButtonReleased(int x, int y, boolean bRight) {
		// easy
		if ((x > 16) && (x < 16 + 22 * 4) && (y > 90 * 4) && (y < 115 * 4)) {
			if (this.player.getCurPosition() == 0) {
				if (!this.player.isRunning()) {
					if (MyTreasureConstants.FIRST_MAP) {
						MyTreasureConstants.FIRST_MAP = false;
						this.tutorial.setVisible(false);
					}
					this.getGame().setLevelChooser(MyTreasureLevelChooser.EASY, 0);
				}
			} else {
				this.player.setNextPosition(0);
			}
		} else if ((x > 43 * 4) && (x < 75 * 4) && (y > 73 * 4) && (y < 103 * 4)) {
			if (this.getGame().getCurSkulls() >= this.NEEDED_SKULLS[0]) {
				if (this.player.getCurPosition() == 2) {
					if (!this.player.isRunning()) {
						this.getGame().setLevelChooser(MyTreasureLevelChooser.MEDIUM, 0);
					}
				} else {
					this.player.setNextPosition(2);
				}
			}
		} else if ((x > 14 * 4) && (x < 39 * 4) && (y > 43 * 4) && (y < 66 * 4)) {
			if (this.getGame().getCurSkulls() >= this.NEEDED_SKULLS[1]) {
				if (this.player.getCurPosition() == 6) {
					if (!this.player.isRunning()) {
						this.getGame().setLevelChooser(MyTreasureLevelChooser.HARD, 0);
					}
				} else {
					this.player.setNextPosition(6);
				}
			}
		} else if ((x > 47 * 4) && (x < 54 * 4) && (y > 16 * 4) && (y < 30 * 4)) {
			if (this.getGame().getCurSkulls() >= this.NEEDED_SKULLS[2]) {
				if (this.player.getCurPosition() == 12) {
					if (!this.player.isRunning()) {
						this.getGame().setLevelChooser(MyTreasureLevelChooser.VERYHARD, 0);
					}
				} else {
					this.player.setNextPosition(12);
				}
			}
		}
	}

	public boolean mouseMoved(int x, int y) {
		if (y > 41) {
			if ((x > 16) && (x < 16 + 22 * 4) && (y > 90 * 4) && (y < 115 * 4)) {
				if (!this.bHandCursor) {
					this.bHandCursor = true;
					this.getGame().getScreen().getComponent().setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				}			
			} else if ((x > 43 * 4) && (x < 75 * 4) && (y > 73 * 4) && (y < 103 * 4)) {
				if (!this.bHandCursor) {
					this.bHandCursor = true;
					this.getGame().getScreen().getComponent().setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				}			
			} else if ((x > 14 * 4) && (x < 39 * 4) && (y > 43 * 4) && (y < 66 * 4)) {
				if (!this.bHandCursor) {
					this.bHandCursor = true;
					this.getGame().getScreen().getComponent().setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				}
			} else if ((x > 47 * 4) && (x < 54 * 4) && (y > 16 * 4) && (y < 30 * 4)) {
				if (!this.bHandCursor) {
					this.bHandCursor = true;
					this.getGame().getScreen().getComponent().setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				}
			} else if (this.bHandCursor) {
				this.bHandCursor = false;
				this.getGame().getScreen().getComponent().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			}
		}
		return false;
	}
	
	public boolean mousePressed(int x, int y, boolean bRight) {
		return false;
	}
	
	@Override
	public void think(int delta) {
		this.player.think(delta);
		
		if (this.tutorial.isVisible()) {
			this.tutorial.think(delta);
			if (!this.tutorial.isVisible()) {
				MyTreasureConstants.FIRST_MAP = false;
			}
		}
		for (int i = this.particles.size() - 1; i >= 0; i--) {
			if (this.particles.get(i).isVisible()) {
				this.particles.get(i).think(delta);
				if (!this.particles.get(i).isVisible()) {
					this.particles.remove(i);
				}
			}
		}
	}

	@Override
	public void render(Graphics2D g) {
		g.drawImage(this.iBackground, 0, 0, null);
		
		g.drawImage(this.getGame().getISheet().getSubimage(0 * 32, 0 * 32, 32, 32), (int)(this.player.getCurX()), (int)(this.player.getCurY()), null);

		g.setColor(MyTreasureConstants.COLOR_DARK);
		g.setFont(MyTreasureConstants.fontMedium);
		String s = String.valueOf(this.getGame().getCurSkulls());
		int w = g.getFontMetrics().stringWidth(s) + 32;
		g.drawString(s, MyTreasureConstants.GAME_WIDTH - 16 - w + 32, 45);
		
		g.drawImage(this.getGame().getISheet().getSubimage(56 * 4, 8 * 4, 32, 32), (int)(MyTreasureConstants.GAME_WIDTH - 16 - w), (int)(20), null);
		
		if ((this.player.getCurPosition() == 0) && (!this.player.isRunning())) {
			g.setColor(MyTreasureConstants.COLOR_LIGHT);
			g.fillRect(4 * 4, 82 * 4, 22 * 4,  8 * 4);
			g.setColor(MyTreasureConstants.COLOR_DARK);
			g.drawRect(4 * 4, 82 * 4, 22 * 4,  8 * 4);
			
			g.setFont(MyTreasureConstants.fontSmall);
			s = MyTreasureConstants.DIFFICULTY[0];
			w = g.getFontMetrics().stringWidth(s);
			g.drawString(s, 15 * 4 - w/2, 87 * 4);
		}
		
		if ((this.player.getCurPosition() == 6) && (!this.player.isRunning())) {
			g.setColor(MyTreasureConstants.COLOR_LIGHT);
			g.fillRect(14 * 4, 30 * 4, 29 * 4,  8 * 4);
			g.setColor(MyTreasureConstants.COLOR_DARK);
			g.drawRect(14 * 4, 30 * 4, 29 * 4,  8 * 4);
			
			g.setFont(MyTreasureConstants.fontSmall);
			s = MyTreasureConstants.DIFFICULTY[2];
			w = g.getFontMetrics().stringWidth(s);
			
			g.drawString(s, 30 * 4 - w/2, 36 * 4);
		} else if (this.getGame().getCurSkulls() < this.NEEDED_SKULLS[1]) {
			g.setColor(MyTreasureConstants.COLOR_DARK);
			s = "need "+this.NEEDED_SKULLS[1];
			g.setFont(MyTreasureConstants.fontVerySmall);
			w = g.getFontMetrics().stringWidth(s);
			g.drawString(s, 29 * 4 - w/2 - 6, 148);

			g.drawImage(MyTreasureConstants.iSheet.getSubimage(56 * 4, 8 * 4, 32, 32), 29 * 4 + w/2 - 6, 148 - 12, 29 * 4 + w/2 - 6 + 12, 148, 0, 0, 32, 32, null);
						
			g.drawImage(MyTreasureConstants.iWays.getSubimage(113, 1, 84, 60), 140, 248, null);
		}
		
		if ((this.player.getCurPosition() == 12) && (!this.player.isRunning())) {
			g.setColor(MyTreasureConstants.COLOR_LIGHT);
			g.fillRect(39 * 4, 6 * 4, 22 * 4,  8 * 4);
			g.setColor(MyTreasureConstants.COLOR_DARK);
			g.drawRect(39 * 4, 6 * 4, 22 * 4,  8 * 4);
			
			g.setFont(MyTreasureConstants.fontSmall);
			s = MyTreasureConstants.DIFFICULTY[3];
			w = g.getFontMetrics().stringWidth(s);
			
			g.drawString(s, 50 * 4 - w/2, 12 * 4);
		} else if (this.getGame().getCurSkulls() < this.NEEDED_SKULLS[2]) {
			g.setColor(MyTreasureConstants.COLOR_DARK);
			s = "need "+this.NEEDED_SKULLS[2];
			g.setFont(MyTreasureConstants.fontVerySmall);
			w = g.getFontMetrics().stringWidth(s);
			g.drawString(s, 51 * 4 - w/2 - 6, 60);

			g.drawImage(MyTreasureConstants.iSheet.getSubimage(56 * 4, 8 * 4, 32, 32), 51 * 4 + w/2 - 6, 60 - 12, 51 * 4 + w/2 - 6 + 12, 60, 0, 0, 32, 32, null);
						
			g.drawImage(MyTreasureConstants.iWays.getSubimage(208, 0, 80, 64), 148, 128, null);
		}
		
		if ((this.player.getCurPosition() == 2) && (!this.player.isRunning())) {
			g.setColor(MyTreasureConstants.COLOR_LIGHT);
			g.fillRect(43 * 4, 65 * 4, 32 * 4,  8 * 4);
			g.setColor(MyTreasureConstants.COLOR_DARK);
			g.drawRect(43 * 4, 65 * 4, 32 * 4,  8 * 4);
			
			g.setFont(MyTreasureConstants.fontSmall);
			s = MyTreasureConstants.DIFFICULTY[1];
			w = g.getFontMetrics().stringWidth(s);
			
			g.drawString(s, 59 * 4 - w/2, 71 * 4);
		} else if (this.getGame().getCurSkulls() < this.NEEDED_SKULLS[0]) {
			g.setColor(MyTreasureConstants.COLOR_DARK);
			s = "need "+this.NEEDED_SKULLS[0];
			g.setFont(MyTreasureConstants.fontVerySmall);
			w = g.getFontMetrics().stringWidth(s);
			g.drawString(s, 59 * 4 - w/2 - 6, 292);

			g.drawImage(MyTreasureConstants.iSheet.getSubimage(56 * 4, 8 * 4, 32, 32), 59 * 4 + w/2 - 6, 292 - 12, 59 * 4 + w/2 - 6 + 12, 292, 0, 0, 32, 32, null);
			
			g.drawImage(MyTreasureConstants.iWays.getSubimage(0, 2, 100, 60), 108, 400, null);
		}
		
		for (int i = this.particles.size() - 1; i >= 0; i--) {
			if (this.particles.get(i).isVisible()) {
				this.particles.get(i).render(g);
			}
		}
		
		if (MyTreasureConstants.FIRST_MAP) {
			this.tutorial.render(g);
		}
	}

}
