/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package myTreasure.game;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import myTreasure.MyTreasureConstants;

public class MyTreasureMenu extends MyTreasureModel {

	public static final String QUIT = "X";
	public static final String GAME = "Play";
	public static final String USERLEVELS = "Userlevels";
	public static final String EDITOR = "Editor";
	public static final String CREDITS = "C";
	
	private BufferedImage iBackground;
	
	public MyTreasureMenu(final MyTreasurePanel game) {
		super(game);
	}

	@Override
	public void init() {
		if (this.iBackground == null) {
			this.iBackground = new BufferedImage(MyTreasureConstants.GAME_WIDTH, MyTreasureConstants.GAME_HEIGHT, BufferedImage.TYPE_INT_RGB);
			Graphics2D g = this.iBackground.createGraphics();
			g.drawImage(this.getGame().getISheet().getSubimage(this.getGame().getISheet().getWidth() - MyTreasureConstants.GAME_WIDTH, 0, MyTreasureConstants.GAME_WIDTH, MyTreasureConstants.GAME_HEIGHT), 0, 0, null);
			
			g.drawImage(this.getGame().getISheet().getSubimage(0, 112 * 4, 72 * 4, 8 * 4), 5 * 4, 25 * 4, null);
			g.drawImage(this.getGame().getISheet().getSubimage(0, 0, 8 * 4, 8 * 4), 5 * 4 + 32 * 4, 25 * 4 - 24, null);

			g.drawImage(this.getGame().getISheet().getSubimage(8 * 32, 1 * 32, 64, 64), 5 * 4 + 50 * 4 - 16, 25 * 4 - 40, null);
			g.drawImage(this.getGame().getISheet().getSubimage(0 * 32, 1 * 32, 32, 32), 5 * 4 + 50 * 4, 25 * 4 - 24, null);

			g.drawImage(this.getGame().getISheet().getSubimage(8 * 32, 1 * 32, 64, 64), 5 * 4 + 12 * 4 - 16, 25 * 4 - 40, null);
			g.drawImage(this.getGame().getISheet().getSubimage(0 * 32, 1 * 32, 32, 32), 5 * 4 + 12 * 4, 25 * 4 - 24, null);
			
			g.setFont(MyTreasureConstants.ORIGINAL_FONT.deriveFont(40f).deriveFont(Font.BOLD));
			g.setColor(new Color(78, 61, 37));
			
			String s = "MyTreasure";
			int w = g.getFontMetrics().stringWidth(s);
			g.drawString(s, MyTreasureConstants.GAME_WIDTH/2 - w/2 + 4, 54);
			
			g.drawImage(this.getGame().getISheet().getSubimage(0, 96 * 4, 76 * 4, 8 * 4), 2 * 4, 390, null);
			
			g.dispose();
		}
	}

	public void keyPressed(int keyCode, char keyCharacter) {
	}
	
	@Override
	public void keyButtonReleased(int button, char character) {
	}

	@Override
	public void mouseButtonFunction(String function) {
		if (function.equals(MyTreasureMenu.QUIT)) {
			this.getGame().saveProperties();
			System.exit(0);
		}
		if (function.equals(MyTreasureMenu.GAME)) {
			this.getGame().setMap();
		}
		if (function.equals(MyTreasureMenu.CREDITS)) {
			this.getGame().setCredits();
		}
		if (function.equals(MyTreasureMenu.EDITOR)) {
			this.getGame().setEditor(false, 95);
		}
		if (function.equals(MyTreasureMenu.USERLEVELS)) {
			this.getGame().setGame(true, false, 0, "");
		}
	}

	@Override
	public void mouseButtonReleased(int x, int y, boolean bRight) {
	}

	public boolean mouseMoved(int x, int y) {
		return false;
	}
	
	public boolean mousePressed(int x, int y, boolean bRight) {
		return false;
	}
	
	@Override
	public void think(int delta) {
	}

	@Override
	public void render(Graphics2D g) {
		g.drawImage(this.iBackground, 0, 0, null);
	}

}
