/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package myTreasure.game;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.util.HashMap;

import myTreasure.MyTreasureComponent;
import myTreasure.MyTreasureConstants;
import myTreasure.entity.MyTreasureIO;
import myTreasure.entity.MyTreasureSolve;
import myTreasure.userlevels.MyTreasureLevels;
import myTreasure.userlevels.MyTreasureUserLevels;

import org.apogames.ApoConstants;
import org.apogames.ApoScreen;

public class MyTreasurePanel extends MyTreasureComponent {

	/** Hilfsvariablen, um herauszufinden wie lange zum Nachdenken und Zeichnen gebraucht wurde */
	private long thinkTime, renderTime;
	
	private MyTreasureModel model;
	
	private MyTreasureMenu menu;

	private MyTreasureMap map;

	private MyTreasureGame game;

	private MyTreasureEditor editor;

	private MyTreasureCredits credits;

	private MyTreasureLevelChooser levelChooser;
	
	private MyTreasureUserLevels userlevels;
	
	private int curSkulls;
	
	private HashMap<String, Integer> solvedLevel;
	
	private MyTreasureIO properties;
	
	private MyTreasureSolve solver;
	
	public MyTreasurePanel(ApoScreen screen) {
		super(screen);
	}
	
	public void init() {
		super.init();
		super.setShowFPS(false);

		if (MyTreasureConstants.iSheet == null) {
			BufferedImage iOriginalBackground = this.getImages().getImage("images/treasure_sheet.png", false);
			MyTreasureConstants.iSheet = new BufferedImage(1024, 512, BufferedImage.TYPE_INT_ARGB_PRE);
			Graphics2D g = MyTreasureConstants.iSheet.createGraphics();
			g.drawImage(iOriginalBackground, 0, 0, MyTreasureConstants.iSheet.getWidth(), MyTreasureConstants.iSheet.getHeight(), 0, 0, iOriginalBackground.getWidth(), iOriginalBackground.getHeight(), null);
			g.dispose();
		}
		if (MyTreasureConstants.iWays == null) {
			BufferedImage iOriginalBackground = this.getImages().getImage("images/ways.png", false);
			MyTreasureConstants.iWays = new BufferedImage(512, 64, BufferedImage.TYPE_INT_ARGB_PRE);
			Graphics2D g = MyTreasureConstants.iWays.createGraphics();
			g.drawImage(iOriginalBackground, 0, 0, MyTreasureConstants.iWays.getWidth(), MyTreasureConstants.iWays.getHeight(), 0, 0, iOriginalBackground.getWidth(), iOriginalBackground.getHeight(), null);
			g.dispose();
		}
		
		MyTreasureButtons buttons = new MyTreasureButtons(this);
		buttons.init();
		
		if (this.menu == null) {
			this.menu = new MyTreasureMenu(this);
		}
		if (this.game == null) {
			this.game = new MyTreasureGame(this);
		}
		if (this.map == null) {
			this.map = new MyTreasureMap(this);
		}
		if (this.editor == null) {
			this.editor = new MyTreasureEditor(this);
		}
		if (this.credits == null) {
			this.credits = new MyTreasureCredits(this);
		}
		if (this.levelChooser == null) {
			this.levelChooser = new MyTreasureLevelChooser(this);
			this.levelChooser.init();
		}
		if (this.solver == null) {
			this.solver = new MyTreasureSolve();
		}
		if (this.userlevels == null) {
			this.userlevels = new MyTreasureUserLevels(this);
			this.loadUserlevels();
		}
		
		this.curSkulls = 0;
		if (this.solvedLevel == null) {
			this.solvedLevel = new HashMap<String, Integer>();
		}
		this.loadProperties();
		
		this.setMenu();
	}

	public void stopGame() {
		this.saveProperties();
	}
	
	public final void loadProperties() {
		if (!ApoConstants.B_APPLET) {
			if (this.properties == null) {
				this.properties = new MyTreasureIO(this);
			}
			try {
				this.properties.readLevel("properties");
			} catch (Exception ex) {
				this.properties.writeLevel("properties");
			}
		}
	}
	
	public final void saveProperties() {
		if (!ApoConstants.B_APPLET) {
			this.properties.writeLevel("properties");
		}
	}
	
	public int getCurSkulls() {
		return this.curSkulls;
	}

	public void setCurSkulls(final int curSkulls) {
		this.curSkulls = curSkulls;
	}

	public HashMap<String, Integer> getSolvedLevel() {
		return this.solvedLevel;
	}
	
	public int getDifficulty() {
		return this.levelChooser.getDifficulty();
	}
	
	public void setOldLevel(final int oldLevel) {
		this.map.setOldLevel(oldLevel);
	}

	public final void setMenu() {
		if (this.model != null) {
			this.model.close();
		}
		this.setShouldRepaint(true);
		
		this.model = this.menu;
		
		this.setButtonVisible(MyTreasureConstants.BUTTON_MENU);
		this.setUserlevelsVisible();
		this.model.init();
		
		this.render();
	}
	
	public final void setMap() {
		if (this.model != null) {
			this.model.close();
		}
		this.setShouldRepaint(true);
		
		this.model = this.map;
		
		this.setButtonVisible(MyTreasureConstants.BUTTON_MAP);
		this.model.init();
		
		this.render();
	}
	
	public final void setLevelChooser(final int difficulty, final int level) {
		if (this.model != null) {
			this.model.close();
		}
		this.setShouldRepaint(true);
		
		this.model = this.levelChooser;
		
		this.setButtonVisible(MyTreasureConstants.BUTTON_LEVELCHOOSER);
		this.model.init();
		
		this.levelChooser.setDifficulty(difficulty, level);
		
		this.render();
	}
	
	public final void setGame(final boolean bUserlevel, final boolean bEditor, final int levelInt, final String levelString) {
		if (this.model != null) {
			this.model.close();
		}
		
		this.setShouldRepaint(true);
		
		this.model = this.game;
		
		this.setButtonVisible(MyTreasureConstants.BUTTON_GAME);

		this.game.setUserLevel(bUserlevel);
		this.model.init();
		
		this.game.loadLevel(bEditor, levelInt, levelString);
	}
	
	public final void setEditor(boolean bSolved, int steps) {
		if (this.model != null) {
			this.model.close();
		}
		this.setShouldRepaint(true);
		
		this.model = this.editor;
		
		this.setButtonVisible(MyTreasureConstants.BUTTON_EDITOR);
		
		this.model.init();
		
		this.editor.setSteps(steps);
		this.editor.setVisibleUpload(bSolved);
		
		this.render();
	}
	
	public final void setCredits() {
		if (this.model != null) {
			this.model.close();
		}
		this.setShouldRepaint(true);
		
		this.model = this.credits;
		
		this.setButtonVisible(MyTreasureConstants.BUTTON_CREDITS);
		
		this.model.init();
		
		this.render();
	}
	
	public int getCurStart() {
		return this.levelChooser.getCurI();
	}
	
	public int getCurMax() {
		return this.levelChooser.getCurMax();
	}
	
	public BufferedImage getISheet() {
		return MyTreasureConstants.iSheet;
	}

	public final void setUserlevelsVisible() {
		if (this.model.equals(this.menu)) {
			if (MyTreasureLevels.editorLevels != null) {
				super.getButtons()[2].setBVisible(true);
			} else {
				super.getButtons()[2].setBVisible(false);
			}
		}
	}
	
	public final void setButtonVisible(boolean[] bVisibile) {
		for (int i = 0; i < this.getButtons().length && i < bVisibile.length; i++) {
			this.getButtons()[i].setBVisible(bVisibile[i]);
		}
		if (ApoConstants.B_APPLET) {
			this.getButtons()[0].setBVisible(false);
		}
		this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
	
	public MyTreasureUserLevels getUserlevels() {
		return this.userlevels;
	}

	public final void loadUserlevels() {
		Thread t = new Thread(new Runnable() {

			public void run() {
				MyTreasurePanel.this.userlevels.loadUserlevels();
			}
 		});
 		t.start();
	}
	
	public final void solveLevel(final String level) {
		Thread t = new Thread(new Runnable() {

			public void run() {
				MyTreasurePanel.this.doSolveLevel(level);
			}
 		});
 		t.start();
	}
	
	private final void doSolveLevel(final String level) {
		String solution = this.solver.solveLevel(level);
		System.out.println(solution);
		if (this.model.equals(this.editor)) {
			this.editor.setSolutionString(solution);
		}
	}
	
	public void save() {
//		this.editor.save();
	}
	
	public void createEmptyLevel(String l) {
//		this.editor.createEmptyLevel(l);
	}

	@Override
	public void keyReleased(final int keyCode, final char keyChar) {
		if (keyCode == KeyEvent.VK_F) {
			super.setShowFPS(!super.isShowFPS());
		}
		if (this.model != null) {
			this.model.keyButtonReleased(keyCode, keyChar);
		}
	}
	
	public void keyPressed(final int keyCode, final char keyCharacter) {
		if (this.model != null) {
			this.model.keyPressed(keyCode, keyCharacter);
		}
	}
	
	@Override
	public void setButtonFunction(final String function) {
		if (this.model != null) {
			this.model.mouseButtonFunction(function);
		}
	}

	@Override
	public boolean mouseReleased(final int x, final int y, final boolean left) {
		if (!super.mouseReleased(x, y, left)) {
			if (this.model != null) {
				this.model.mouseButtonReleased(x, y, !left);
			}
		}
		return false;
	}
	
	public void mousePressed(final int x, final int y, final boolean left) {
		super.mousePressed(x, y, left);		
		if (this.model != null) {
			this.model.mousePressed(x, y, !left);
		}
	}
	
	public void mouseMoved(final int x, final int y) {
		super.mouseMoved(x, y);
		if (this.model != null) {
			this.model.mouseMoved(x, y);
		}
	}
	
	public void mouseDragged(final int x, final int y, final boolean bLeft) {
		super.mouseDragged(x, y, bLeft);
		if (this.model != null) {
			this.model.mouseDragged(x, y, !bLeft);
		}
	}
	
	@Override
	public void mouseWheelChanged(final int changed) {
		if (this.model != null) {
			this.model.mouseWheelChanged(changed);
		}
	}
	
	public void think(final long delta) {
		long t = System.nanoTime();
		
		if (this.model != null) {
			this.model.think((int)delta);
		}
		this.thinkTime = (int) (System.nanoTime() - t);
	}

	@Override
	protected void render(final Graphics2D g) {
		long t = System.nanoTime();

		if (this.model != null) {
			g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			this.model.render(g);
		}
		super.renderButtons(g);
		if (this.model != null) {
			this.model.drawOverlay(g);
		}
		this.renderFPS(g);
		this.renderTime = (int)(System.nanoTime() - t);
	}

	
	/**
	 * rendert die Anzeige fr die FPS und die Thinkzeit
	 * @param g : das Graphicsobjekt
	 */
	private void renderFPS(final Graphics2D g) {
		if (super.isShowFPS()) {
			g.setColor(Color.red);
			g.setFont(MyTreasureConstants.FONT_FPS);
			g.drawString("think time: " + this.thinkTime + " ns", 5, MyTreasureConstants.GAME_HEIGHT - 45);
			g.drawString("think time: " + (this.thinkTime / 1000000) + " ms",5, MyTreasureConstants.GAME_HEIGHT - 35);

			g.drawString("draw time: " + this.renderTime + " ns", 5, MyTreasureConstants.GAME_HEIGHT - 25);
			g.drawString("draw time: " + (this.renderTime / 1000000) + " ms", 5, MyTreasureConstants.GAME_HEIGHT - 15);
			g.drawString("FPS: " + super.getFPS(), 5, MyTreasureConstants.GAME_HEIGHT - 5);
		}
	}
	
	public void renderBackgroundInfo(Graphics2D g) {
		
	}

}
