/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.apogames;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;

/**
 * abstrakte Klasse fr das Einlesen und Speichern von Daten
 * @author Dirk Aporius
 */
public abstract class ApoIO{
	
	/**
	 * Konstruktor
	 */
	public ApoIO() {
		super();
	}

	/**
	 * schreibt eine Datei in das bergebene String Filesystem
	 * @param fileName = wo wird hingespeichert
	 */
	@SuppressWarnings("resource")
	public void writeLevel(String fileName) {
		try	{
			DataOutputStream data = null;
			
			if (!ApoConstants.B_APPLET) {
				FileOutputStream file = new FileOutputStream(fileName);
				BufferedOutputStream buff = new BufferedOutputStream(file);
				data = new DataOutputStream(buff);
			} else {
				URL connectionURL = new URL(fileName);
				HttpURLConnection connection = (HttpURLConnection)connectionURL.openConnection(); 
				connection.setDoOutput(true);
				data = new DataOutputStream(connection.getOutputStream());
			}
			
			if (data != null) {
				this.writeLevel(data);
				data.close();
			}
			
		} catch (IOException e) {
			System.out.println("Error: "+e);
		}
	}
	
	/**
	 * abstrakte Methode die genutzt werden kann um Daten zu schreiben
	 * @param data = das DataOutputStreamObjekt mithilfe dessen man in die Datei schreiben kann
	 * @return TRUE falls alles glatt lief sonst FALSE
	 * @throws IOException = Exception die geworfen wird, falls etwas beim Schreiben schief gegangen ist
	 */
	public abstract boolean writeLevel(DataOutputStream data) throws IOException;
	
	/**
	 * liest eine Datei ein
	 */
	public boolean readLevel(String fileName) {
		return this.readLevel(ApoConstants.B_APPLET, fileName);
	}

	/**
	 * liest eine Datei ein
	 * @param bURL: TRUE = Level aus dem Internet laden, sonst von der Festplatte
	 */
	@SuppressWarnings("resource")
	public boolean readLevel(boolean bURL, String fileName) {
		try {
			DataInputStream data;
			if (!bURL) {
				FileInputStream file		= new FileInputStream(fileName);
				BufferedInputStream buff	= new BufferedInputStream(file);
				data = new DataInputStream(buff);
			} else {
				URL connection = new URL(fileName);
				data = new DataInputStream(connection.openStream());
			}
			try	{
				this.readLevel( data );
			} catch ( EOFException e ) {
			}
			data.close();
		} catch ( IOException e ) {
			System.out.println("Error: "+e);
			return false;
		}
		return true;
	}
	
	/**
	 * abstrakte Methode, um eine Datei einzulesen
	 * @param data = das DataInputStreamObjekt mithilfe dessen man die Datei lesen kann
	 * @return TRUE, falls alles glatt gegangen ist, sonst FALSE
	 * @throws EOFException = Exception die geworfen wird falls das Ende der einzulesenden Datei erreicht ist und man trotzdem noch was lesen mchte
	 * @throws IOException = Exception die geworfen wird, falls etwas beim Einlesen der Datei schiefgeht
	 */
	public abstract boolean readLevel(DataInputStream data) throws EOFException, IOException;
}
