/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.apogames;

import org.apogames.ApoScreen;
import org.apogames.ApoSubGame;


/**
 * Spielstartklasse
 * @author Dirk Aporius
 *
 */
public final class ApoLibraryGame extends Thread {

    /** SubGame ID fr das Spiel */
    public static final int GAME = 0;
    /** SubGame ID frs Beenden des Spiels */
    public static final int EXIT = -1;

    /** nchstes SubGame wird mit dieser Variable gesetzt */
    private int nextGameID = ApoLibraryGame.GAME;
    /** Screenobjekt des Spiel */
    private ApoScreen screen = null;
    
    private final ApoSubGame game;

    /**
     * Konstruktor
     * @param title : Titel des Spiels
     * @param displayConfiguration : Displaykonfigurationen
     */
    public ApoLibraryGame(final ApoSubGame game) {
        super();
        this.game = game;
        this.screen = this.game.getScreen();//new ApoScreen(title, displayConfiguration);
    }

    /** Spiel startet hier */
    @Override
    public void run() {
        while (this.nextGameID != -1) {
            try {
                ApoSubGame subGame = selectGame(this.nextGameID);
                if (subGame == null) {
                    this.nextGameID = -1;
                } else {
                    subGame.start();
                    subGame.join();
                    this.nextGameID = subGame.getNextID();
                }
            } catch (InterruptedException e) {
            }
        }
    }
    
    /**
     * gibt den eigentlichen Screen des Spiels zurck
     * @return gibt den eigentlichen Screen des Spiels zurck
     */
    public final ApoScreen getScreen() {
    	return this.screen;
    }

	/**
	 * mit dieser Methode kann zwischen den einzelnen Subgames hin und hergeschaltet werden
	 * @param gameID : GameID fr das jeweilige SubGameObjekt
	 * @return das neue SubGameObjekt
	 * @throws InterruptedException
	 */
    private ApoSubGame selectGame(int gameID) throws InterruptedException {
        switch (gameID) {
            case ApoLibraryGame.GAME:	return this.game;
            default:					return null;
        }
    }
}
