/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.apogames.help;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;

import org.apogames.ApoConstants;

/**
 * Klasse, um eine Class Datei im laufenden Betrieb zu laden
 * @author Dirk Aporius
 *
 */
public class ApoClassLoader {
	private String path;
	private String classname;

	public ApoClassLoader(String path, String classname) {
		this.setPath( path );
		this.setClassname( classname );
	}

	/**
	 * gibt den Klassennamen zurck
	 * @return gibt den Klassennamen zurck
	 */
	public String getClassname() {
		return this.classname;
	}

	/**
	 * setzt den Klassennamen auf den bergebenen Wert
	 * @param classname
	 */
	public void setClassname(String classname) {
		this.classname = classname;
	}

	/**
	 * gibt den Pfad zur Klasse zurck
	 */
	public String getPath() {
		return this.path;
	}

	/**
	 * setzt den Pfad, welcher zur Klasse fhrt, auf den
	 * bergebenen Wert
	 * @param path
	 */
	public void setPath(String path) {
		this.path = path;
	}

	/**
	 * liefert ein Object der Klasse zurck, welches nur noch in die
	 * entsprechende Klasse gecastet werden muss
	 * @return liefert ein Object der Klasse zurck, welches nur noch in die
	 * entsprechende Klasse gecastet werden muss
	 */
	public Object getMyClass() { 
		try {
			URL url;
			if ( !ApoConstants.B_APPLET )
				url = new File( this.getPath() ).toURI().toURL();
			else
				url = new URL( this.getPath() );
			URLClassLoader cl = new URLClassLoader( new URL[] { url } ); 
			Class<?> c;
			try {
				//System.out.println( this.getPath() + "" + this.getClassname() );
				c = cl.loadClass( this.getClassname() );
			    try {
					return c.newInstance();
				} catch (InstantiationException e) {
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				}
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} 

		return null;
	  } 

}