/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.apogames.help;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * Klasse, um Dateien oder ganze Ordner einfach zu kopieren
 * @author Dirk Aporius
 *
 */
public class ApoCopy {

	/**
	 * Konstruktor
	 */
	public ApoCopy() {
	}
	
	/**
	 * kopiert ein komplettes Verzeichnis (ohne Unterverzeichnisse)
	 * in einen bergebenen Bereich
	 * @param pSrcDirectory = String mit Pfadangabe zum Ordner, welcher kopiert werden 
	 * @param pCopyDirectory = String mit Pfadangabe zum Ordner, wohin kopiert werden soll
	 */
	public void copyDirectory( String pSrcDirectory, String pCopyDirectory ) {
		File srcDirectory = new File(pSrcDirectory);
		if (!srcDirectory.exists()) {
			return;
		}
		File copyDirectory = new File( pCopyDirectory );
		if (!copyDirectory.exists()) {
			copyDirectory.mkdirs();
		}
		File[] files = srcDirectory.listFiles();
		for ( int i = 0; i < files.length; i++ ) {
			if (!files[i].isDirectory()) {
				String fileName = files[i].getPath().substring(files[i].getPath().lastIndexOf(File.separator));
				if(!new File(pCopyDirectory + File.separator + fileName).exists()) {
					this.copyFile(files[i].getPath(), pCopyDirectory + File.separator + fileName);
				}
			}
		}
	}
	
	/**
	 * kopiert eine Datei in ein anderes bergebenes Verzeichnis
	 * @param pSrcFile = String mit Pfadangabe zur Datei, welche kopiert werden soll
	 * @param pCopyFile = String mit Pfadangabe zur Datei, wohin kopiert werden soll
	 */
	public void copyFile( String pSrcFile, String pCopyFile ) {
		FileInputStream fis = null;
		FileOutputStream fos = null;
		
		try {
			fis = new FileInputStream(pSrcFile);
			fos = new FileOutputStream(pCopyFile);
			
			byte[] buffer = new byte[0xFFFF];
			for ( int len; (len = fis.read(buffer)) != -1; ) {
				fos.write(buffer, 0, len);
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if ( fis != null ) {
				try { fis.close(); } catch (IOException e) { e.printStackTrace(); }
			}
			if ( fos != null ) {
				try { fos.close(); } catch (IOException e) { e.printStackTrace(); }
			}
		}
	}
	
}
