/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.apogames.help;

import java.io.File;

/**
 * Klasse, um im OpenDialog nur bestimmte Dateien zu sehen und
 * nicht alle anderen Dateiformate
 * @author Dirk Aporius
 *
 */
public class ApoFileFilter extends javax.swing.filechooser.FileFilter 
{
	String s	= "";
	
	String[] aString;
	
	public ApoFileFilter( String s )
	{
		this.s		= s;
	}
	
	public ApoFileFilter( String[] aString ) {
		this.s = null;
		this.aString	= aString;
	}
	
	public String getLevelName(){
		if (this.s != null) {
			return "."+this.s;	
		} else if (this.aString != null) {
			String s = "";
			for (int i = 0; i < this.aString.length; i++) {
				s += this.aString[i];
			}
			return s;
		}
		return "";
	}
	
    public String getExtension(File file) 
    {
        String s = "";
        String s1 = file.getName();
        int i = s1.lastIndexOf(46);
        if(i > 0 && i < s1.length() - 1)
            s = s1.substring(i + 1).toLowerCase();
        return s;
    }

    /**
     * gibt zurck, welches Dateien angezeigt werden sollen
     * @return gibt zurck, welches Dateien angezeigt werden sollen
     */
    public String getDescription() 
    {
    	if ( this.s != null ) {
    		return "Nur "+this.s+"-Dateien";
    	} else if ( this.aString != null ) {
    		String s = "Nur ";
    		for ( int i = 0; i < this.aString.length; i++ ) {
    			s += this.aString[i];
    			if (i < this.aString.length - 1) {
    				s += ", ";
    			}
    		}
    		s += " Dateien";
    		return s;
    	}
    	return "";
    }

    /**
     * akzeptiert nur die Dateien, die er annehmen soll, alle anderen nimmt er nicht an
     */
    public boolean accept(File file) 
    {
        if(file.isDirectory())
            return true;
        String s = getExtension(file);
        if(s != null) {
        	if ( this.aString != null ) {
        		for ( int i = 0; i < this.aString.length; i++ ) {
        			if ( s.equals( this.aString[i] ) )
        				return true;
        		}
        	}
            return ((s.equals(this.s)));
        } else {
            return false;
        }
    }
        
    public ApoFileFilter() 
    {
    }
}
