/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.apogames.sound;

//Ralph-Uwe RYll 2010 Version 1.0
import java.io.IOException;
import java.net.URL;
import java.sql.Time;
import java.util.TimeZone;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.FloatControl;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;

public class ApoMP3Sound implements Runnable
{
	private Thread runner = new Thread(this); //AbspielThread
	private static int BufferSize = 1024; // Anzahl der Daten, die aufeinmal an die Soundkarte geschickt werden.
    private static byte[]  buffer = new byte[BufferSize];
    private int gainPercent = 90;  //gibt die Lautstrke in Prozent an.  (0% = -80dB und 100% = 6dB) 
	private Boolean stop = false;
	private Boolean loopPlay = false;
	private URL song = null;
	private static long timeZoneKorrektur = TimeZone.getDefault().getOffset(0);
	private Time time = new Time(-timeZoneKorrektur);
	private Time songTime = new Time(-timeZoneKorrektur);
	private int sampleSizeInBits = 0;
	private long songLaenge = 0;
	private boolean reset = false;
	private Boolean isPlaying = false;
	private long resetKorrektur = 0;
	private boolean pause = false;
	private boolean mute = false;
	private int lautstaerke = gainPercent;

	/**
	 * damit mp3-Dateien abgespielt werden knnen, muss das mp3plugin von Sun im Classpath stehen oder
	 * (zB. unter Eclipse) als Bibliothek eingetragen sein.  
	 * Das gleiche gilt fr flac-Dateien.
	 * 
	 * Die Lautstrke "gainLevel" ist logarhytmisch von -80dB bis ca. 6dB. Daher ist die 
	 * prozentuale Lautstrkeregelung nicht ganz korrekt. 
	 */
	public ApoMP3Sound() {
		
	}
	
	public ApoMP3Sound(String song) {
		this.setSong(song);
	}
	
	/**
	 * gibt die aktuelle Zeit an
	 */
	public Time getCurrentTime() {
		return time;
	}



	/**
	 * gibt zurck, ob das Musikstck wiederholt wird
	 */
	public boolean isLoopPlay(){
		return loopPlay;
	}
	
	/**
	 * startet eine Endlosschleife
	 *
	 */
	public void setLoopPlay(Boolean loop){
		loopPlay = loop;
	}
	
	/**
	 * stoppt die Wiedergabe
	 *
	 */
	public void stop()
	{
		stop=true;
	}
	
	/**
	 * starten der Wiedergabe
	 *
	 */
	public void play()
	{
			stop=false;
			if (!runner.isAlive())
			{
				runner = new Thread(this);
				runner.start();
			}
	}
	
	/**
	 * gibt die aktuelle Lautstrke zurck
	 */
	public int getVolume(){
		return gainPercent;
	}
	
	/**
	 * Wert zwischen 0% und 100%
	 * @param volumen
	 */
	public void setVolumen(int volumen){
		if ((volumen <= 100) || (volumen >= 0)){
			gainPercent = volumen;
		}
		
	}
	
	
	/**
	 * der WiedergabeThread
	 */
	public void run() 
	{
		do{
		try {
			resetKorrektur = 0;
			AudioInputStream in = AudioSystem.getAudioInputStream(AudioFormat.Encoding.PCM_SIGNED, AudioSystem.getAudioInputStream(song));
			AudioFormat audioFormat = in.getFormat();
			SourceDataLine line = (SourceDataLine) AudioSystem.getLine(new DataLine.Info(SourceDataLine.class, audioFormat));
		    line.open(audioFormat);
		    FloatControl gainControl = (FloatControl)line.getControl(FloatControl.Type.MASTER_GAIN);
		    line.start();
		    //songLaenge = song.length();
		    sampleSizeInBits = audioFormat.getSampleSizeInBits();	
		    
		    if (in.getFrameLength() == -1){
		    	songTime.setTime(songLaenge/sampleSizeInBits - timeZoneKorrektur);
		    }
		    else{
		    	songTime.setTime((in.getFrameLength()/(long)audioFormat.getFrameRate())*1000 - timeZoneKorrektur);
		    }
		    
		    in.mark(in.available());
		    while ((true)&&(!stop)){
		    	isPlaying = true;
		    	int gainLevel= (int) ((int)gainControl.getMinimum()+((gainControl.getMaximum()-gainControl.getMinimum())/100*gainPercent));
		    	gainControl.setValue(gainLevel);
		    	if (!pause){
			    	int n = in.read(buffer, 0, buffer.length);
			    	if ((n < 0)||(stop)) {
			        	break;
			        }
			    	if (reset) {
			    		resetKorrektur =  line.getMicrosecondPosition()/1000;
			    		in.reset();
			    		reset = false;
			    	}
			    	time.setTime(line.getMicrosecondPosition()/1000 - timeZoneKorrektur - resetKorrektur);
			    	line.write(buffer, 0, n);
		    	}
		      }
		      line.drain();
		      line.close();
		      in.close();
			} catch (UnsupportedAudioFileException e) {
			    System.out.println("nicht untersttztes Format");
			} catch (IOException e) {
				System.out.println("Datei nicht gefunden" + e );
			} catch (LineUnavailableException e) {
				System.out.println("Soundkartenfehler");
			}
		}while (loopPlay && !stop);
		isPlaying = false;
	}
	
	/**
	 * Name und Pfad der Sounddatei
	 * @param song
	 */
	public void setSong(String song) {
		this.song = this.getClass().getResource(song);
		
	}

	/** 
	 * Name und Pfad der Sounddatei
	 * @return
	 */
	public URL getSong() {
		return song;
	}

	/**
	 * die Gesamtlnge des Musikstcks
	 * @return
	 */
	public Time getSongTime() {
		return songTime;
	}

	/**
	 * setzt den aktuellen Titel zurck
	 * @param reset
	 */

	public void reset(boolean reset) {
		this.reset = reset;
	}
	/**
	 * gibt zurck, ob gerade ein Titel abgespielt wird
	 * @return
	 */
	public Boolean isPlaying() {
		return isPlaying;
	}
	

	/**
	 * Titel pausieren
	 * @param pause
	 */
	public void setPause(boolean pause) {
		this.pause = pause;
	}


	/**
	 * gibt zurck, ob der Titel pausiert
	 * @return
	 */
	public boolean isPause() {
		return pause;
	}


	/**
	 * schaltet die Wiedergabe stumm
	 * @param mute
	 */
	public void setMute(boolean mute) {
		if ((mute) &&(!this.mute)){
			lautstaerke = this.getVolume();
			this.setVolumen(0);
		}else{
			this.setVolumen(lautstaerke);
		}
		this.mute = mute;
	}


	/**
	 * gibt zurck, ob die Wiedergabe stumm geschaltet ist
	 * @return
	 */
	public boolean isMute() {
		return mute;
	}

	


}