/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.apogames.sound;

import java.util.ArrayList;

import javax.sound.midi.MidiChannel;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.Sequence;
import javax.sound.midi.Sequencer;
import javax.sound.midi.Synthesizer;

public class ApoMidi {

	private ArrayList<Sequence> songs = new ArrayList<Sequence>();
	private Sequencer sequencer;
	
	public void addSong(String s) {
		try {
			if (this.sequencer == null) {
				this.sequencer = MidiSystem.getSequencer();
				this.sequencer.open();
			}
			this.songs.add(MidiSystem.getSequence(ApoMidi.class.getResourceAsStream(s)));
		} catch (Exception ex) {
			this.sequencer = null;
			this.songs.clear();
			System.out.println("Ex mit "+ex.getMessage());
		}
	}

	public void startMusic(int song) {
		this.stopMusic();
		if ((song < 0) || (song > this.songs.size()) || (this.songs.size() <= 0)) {
			return;
		}
		if (this.sequencer != null) {
			try {
				this.sequencer.open();
				this.sequencer.setSequence((Sequence) null);
				this.sequencer.setSequence(this.songs.get(song));
				this.sequencer.setLoopCount(Sequencer.LOOP_CONTINUOUSLY);
				this.sequencer.start();
			} catch (Exception e) {
			}
		}
	}

	public void stopMusic() {
		if (this.sequencer != null) {
			try {
				this.sequencer.stop();
				this.sequencer.close();
			} catch (Exception e) {
			}
		}
	}

	public void setVolume(int volume) {
		try {
			if (this.sequencer instanceof Synthesizer) {
				MidiChannel[] channels = ((Synthesizer)(this.sequencer)).getChannels();
				double gain = (float) volume / 100f;
				for (int i = 0; i < channels.length; i++) {
					channels[i].controlChange(7, (int) (gain * 127.0));
				}
				System.out.println("Check");
			}
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}
}
